import base64
import requests
import json, uuid
def get_voice(text,root_path, langeCode='vi-VN',name=None,effect="telephony-class-application"):
    try:
        if name==None:
            name=f"{langeCode}-Wavenet-B"
        headers = {
            'x-origin': 'https://explorer.apis.google.com',
            'content-type': 'application/json',
            'Content-Type': 'text/plain',
        }

        params = (
            ('key', 'AIzaSyAa8yy0GdcGPHdtD083HiGGx_S0vMPScDM'),
            ('alt', 'json'),
        )

        data_request = {
            "input": {
                "text": text
            },
            "voice": {
                "languageCode": langeCode,
                "name": name
            },
            "audioConfig": {
                "audioEncoding": "MP3_64_KBPS",
                "pitch": 0,
                "speakingRate": 1,
                "effectsProfileId": [effect]
            }
        }

        response = requests.post('https://content-texttospeech.googleapis.com/v1beta1/text:synthesize', headers=headers, params=params, json=data_request)

        data = json.loads(response.text)
        base64_message = data['audioContent']

        base64_bytes = base64_message.encode('ascii')
        message_bytes = base64.b64decode(base64_bytes)
        str(uuid.uuid4())
        path_voice = f"{root_path}/voice-{str(uuid.uuid4())}.mp3"
        with open(path_voice, 'wb') as file_to_save:
            file_to_save.write(message_bytes)
    except:
        pass
        return None
    return path_voice


import os, json
import uuid,math,shutil
def create_vid_text_img(img_bg, img_text, root_path, type=0, w=1280, h=720):

    final_path = f"{root_path}/txt-on-img-vid-{str(uuid.uuid4())}.mp4"
    cmddf = f'ffmpeg -y -loop 1 -i "{img_bg}" -t 10 -r 1 -loop 1 -i "{img_text}" -filter_complex " scale=w=3*{w}:h=3*{h}, ' \
          f'zoompan=z=min(max(zoom\,pzoom)+0.0008\,1.5):d=250:s={w}x{h}, setsar=1 [bg]; [bg][1:v]overlay=0:500:shortest=1,format=yuv420p[v1];[v1] fade=in:0:30[v] "' \
          f' -map "[v]" -c:v h264 -crf 18 -preset veryfast "{final_path}"'
    if type==0:
        # zoomIn
        cmd = f'ffmpeg -y -loop 1 -i "{img_bg}" -t 10 -r 1 -loop 1 -i "{img_text}" -filter_complex " scale=w=3*{w}:h=3*{h}, ' \
            f'zoompan=z=min(max(zoom\,pzoom)+0.0008\,1.5):d=250:s={w}x{h}, setsar=1 [bg]; [bg][1:v]overlay=0:500:shortest=1,format=yuv420p[v1];[v1] fade=in:0:30[v] "' \
            f' -map "[v]" -c:v h264 -crf 18 -preset veryfast "{final_path}"'
    if type==1:
        # zoomOut
        cmd = f'ffmpeg -y -loop 1 -i "{img_bg}" -t 10 -r 1 -loop 1 -i "{img_text}" -t 10 -filter_complex "[0:v] scale=w=3*{w}:h=3*{h}, zoompan=z=if(lte(zoom\,1.0)\,1.3\,max(1.0\,zoom-0.0008)):d=250:s={w}x{h}, setsar=1 [bg] ; [bg][1:v]overlay=0:500:shortest=1,format=yuv420p[v1];[v1] fade=in:0:30[v] " -map "[v]" -c:v h264 -crf 18 -preset veryfast "{final_path}"'
    if type==2:
        # rotateCW
        cmd=f'ffmpeg -y -loop 1 -i "{img_bg}" -t 10 -r 1 -loop 1 -i "{img_text}" -t 10 -filter_complex " [0:v] scale=w=\'if( gt(iw, ih), -2, 2202.9071700823 )\':h=\'if( gt(iw,ih), 2202.9071700823, -2 )\', rotate=a=0.39269908169872*t/10:c=black:ow={w}:oh={h}, setsar=1 [bg]; [bg][1:v]overlay=0:500:shortest=1,format=yuv420p[v1];[v1] fade=in:0:30[v] " -map "[v]" -c:v h264 -crf 18 -preset veryfast "{final_path}"'
        cmd=cmddf
    if type==3:
        # rotateCCW
        cmd=f'ffmpeg -y -loop 1 -i "{img_bg}" -t 10 -r 1 -loop 1 -i "{img_text}" -t 10 -filter_complex " [0:v] scale=w=\'if( gt(iw, ih), -2, 2202.9071700823 )\':h=\'if( gt(iw,ih), 2202.9071700823, -2 )\', rotate=a=-0.39269908169872*t/10:c=black:ow={w}:oh={h}, setsar=1 [bg]; [bg][1:v]overlay=0:500:shortest=1,format=yuv420p[v1];[v1] fade=in:0:30[v] " -map "[v]" -c:v h264 -crf 18 -preset veryfast "{final_path}"'
        cmd=cmddf
    if type==4:
        # panUp
        cmd=f'ffmpeg -y -loop 1 -i "{img_bg}" -t 10 -r 1 -loop 1 -i "{img_text}" -t 10 -filter_complex " [0:v] scale=w=3*{w}:h=3*{h} , crop=w=3*{w}/1.2:h=3*{h}/1.2:y=t*(in_h-out_h)/10, scale=w={w}:h={h}, setsar=1 [bg] ; [bg][1:v]overlay=0:500:shortest=1,format=yuv420p[v1];[v1] fade=in:0:30[v] " -map "[v]" -c:v h264 -crf 18 -preset veryfast "{final_path}"'
    if type==5:
        # panDown
        cmd=f'ffmpeg -y -loop 1 -i "{img_bg}" -t 10 -r 1 -loop 1 -i "{img_text}" -t 10 -filter_complex " [0:v] scale=w=3*{w}:h=3*{h} , crop=w=3*{w}/1.2:h=3*{h}/1.2:y=(in_h-out_h)-t*(in_h-out_h)/10, scale=w={w}:h={h}, setsar=1 [bg] ; [bg][1:v]overlay=0:500:shortest=1,format=yuv420p[v1];[v1] fade=in:0:30[v] " -map "[v]" -c:v h264 -crf 18 -preset veryfast "{final_path}"'
    if type==6:
        # panLeft
        cmd=f'ffmpeg -y -loop 1 -i "{img_bg}" -t 10 -r 1 -loop 1 -i "{img_text}" -t 10 -filter_complex " [0:v] scale=w=3*{w}:h=3*{h} , crop=w=3*{w}/1.1:h=3*{h}/1.1:x=t*(in_w-out_w)/10, scale=w={w}:h={h}, setsar=1 [bg] ; [bg][1:v]overlay=0:500:shortest=1,format=yuv420p[v1];[v1] fade=in:0:30[v] " -map "[v]" -c:v h264 -crf 18 -preset veryfast "{final_path}"'
    if type==7:
        # panRight
        cmd=f'ffmpeg -y -loop 1 -i "{img_bg}" -t 10 -r 1 -loop 1 -i "{img_text}" -t 10 -filter_complex " [0:v] scale=w=3*{w}:h=3*{h} , crop=w=3*{w}/1.1:h=3*{h}/1.1:x=(in_w-out_w)-t*(in_w-out_w)/10, scale=w={w}:h={h}, setsar=1 [bg] ; [bg][1:v]overlay=0:500:shortest=1,format=yuv420p[v1];[v1] fade=in:0:30[v] " -map "[v]" -c:v h264 -crf 18 -preset veryfast "{final_path}"'
    os.system(cmd)
    return final_path




import requests,json
from lxml import html
import re, textwrap
def crawl_rss():
    print("crawl rss")
def crawl_data_slideshow(url):
    print("crawl data sideshow")
    try:
        page = requests.get(url, headers={"user-agent": "Googlebot-News/1.0"})
        tree = html.fromstring(page.content)
        arr_img=[]
        arr_text=[]
        title = tree.xpath('//title')[0].text
        arr_img_obj = tree.xpath('//ul[contains(@class,"slideshow")]/li//img[@data-record-deferred-loadtime]')
        if arr_img_obj is not None:
            for img in arr_img_obj:
                is_show=True
                data_src = json.loads(img.get("data-src"))
                try:
                    src = data_src['default']['src']
                except:
                    src = data_src['default']
                title=img.get("title").split(" - ")[0]
                title=title.strip().strip(".")
                body=img.get("alt").strip()
                body=re.sub('^(.*?):','',body).strip()
                if title not in body:
                    body=title+", \n"+body
                arr_text.append(textwrap.shorten(body,300, placeholder="..."))
                arr_img.append("http:" + src.split("?")[0])
    except:
        pass
    return arr_text, arr_img, title
def crawl_data(url):
    print("crawl data")
    try:
        page = requests.get(url, headers={"user-agent": "Googlebot-News/1.0"})
        tree = html.fromstring(page.content)
        title=tree.xpath('//title')[0].text
        arr_text_obj = tree.xpath('//div[@id="maincontent"]//p[not (descendant::img or descendant::a)]')
        arr_text= []
        for text in arr_text_obj:
            if text is not None and text.text is not None:
                #print(text.text)
                arr_text.append(text.text.strip())
        arr_img_obj = tree.xpath('//div[@id="maincontent"]//span/img/@data-src')
        arr_img=[]
        if arr_img_obj is not None:
            for img in arr_img_obj:
                data_src = json.loads(img)
                try:
                    src=data_src['default']['src']
                except:
                    src=data_src['default']
                arr_img.append("http:"+src.split("?")[0])
        #print(html.tostring(tree))
        is_show=False
        arr_img_obj = tree.xpath('//ul[contains(@class,"slideshow")]/li//img/@data-src')
        if arr_img_obj is not None:
            for img in arr_img_obj:
                is_show=True
                data_src = json.loads(img)
                try:
                    src = data_src['default']['src']
                except:
                    src = data_src['default']
                arr_img.append("http:" + src.split("?")[0])
        if is_show:
            arr_text_obj = tree.xpath('//div[@id="maincontent"]//div[@class="body-text"]//h2')
            for text in arr_text_obj:
                if text is not None and text.text is not None:
                    # print(text.text)
                    arr_text.append(text.text.strip())
    except:
        pass
    return arr_text, arr_img, title




import re
import collections,os
from anews.common import drawhelper as dr
import uuid
from slugify import slugify
class Letter():
    def __init__(self,text,value):
        self.text=text;
        self.value=value;
    def __str__(self):
        return self.text+":"+str(self.value)

def words(text,regex):
    pattern = re.compile(regex)
    for match in pattern.finditer(text):
        yield match.group()

def phrases(words, xlen=2):
    phrase = []
    for word in words:
        phrase.append(word)
        if len(phrase) > xlen:
            phrase.remove(phrase[0])
        if len(phrase) == xlen:
            yield tuple(phrase)


def common_data(list1, list2):
    result = False

    # traverse in the 1st list
    for x in list1:

        # traverse in the 2nd list
        for y in list2:

            # if one common
            if x == y:
                result = True
                return result

    return result

def make_tag(title,text, lang):
    regex = r"[^\s]+"
    spacing = " "
    if (lang == 'cn'):
        regex = r"[^\x00-\x7F]"
        spacing = ""
    counts = collections.defaultdict(int)
    list_title = list(phrases(words(title, regex)))
    list_content = list(phrases(words(text, regex)))
    list_total = list_title + list_content
    for phrase in list_total:
        counts[phrase] += 1
    counts2=sorted(counts.items(),key=lambda i:i[1],reverse=True)
    i=0
    arr_tag=[]
    max_tag=30
    while i <len(counts2)-1 and len(arr_tag)<max_tag:
        if counts2[i][1] == counts2[i + 1][1] and common_data(counts2[i][0], counts2[i + 1][0]):
            arr_tag.append(" ".join(counts2[i][0]) + " " + counts2[i + 1][0][1])
            i += 1
        else:
            arr_tag.append(" ".join(counts2[i][0]))
        i+=1
    return ",".join(arr_tag)

def make_title(title,text,image_path,font_path, font_size, lang, tmp_path, logo_path):
    regex=r"[^\s]+"
    spacing=" "
    if(lang=='zh' or lang =='ja'):
        regex=r"[^\x00-\x7F]"
        spacing=""

    output = os.path.join(tmp_path, slugify(title) + "_logo_text.jpg")
    counts = collections.defaultdict(int)
    list_title = list(phrases(words(title,regex)))
    list_content = list(phrases(words(text,regex)))
    list_total = list_title + list_content
    for phrase in list_total:
        counts[phrase] += 1
    list_words1 = {}
    list_words2 = {}
    list_words2[list_title[0][0]] = 1
    for phrase in list_title:
        list_words1[phrase[0]] = counts.get(phrase)
        list_words2[phrase[1]] = counts.get(phrase)
    list_words1[list_title[len(list_title) - 1][1]] = 1
    list_words_total = []
    #find top value
    arr_v = []
    for v in list_words1.values():
        if v not in arr_v:
            arr_v.append(v)
    arr_v.sort(reverse=True)
    for w in words(title, regex):
        list_words_total.append(Letter(w, max(list_words1.get(w), list_words2.get(w))))
    out_put_text = dr.write_text_on_logo(image_path, tmp_path,
                          list_words_total , arr_v,
                          font_path , None, font_size, spacing)
    if logo_path:
        dr.add_logo(out_put_text,logo_path,tmp_path,output)
        os.remove(out_put_text)
    else:
        output=out_put_text
    return output